"use client";

import {
  FiImage,
  FiEdit2,
  FiChevronRight,
  FiChevronLeft,
  FiTool,
  FiGlobe,
} from "react-icons/fi";
import { FeedbackType } from "../types";
import { useContext, useEffect, useRef, useState } from "react";
import ReactMarkdown from "react-markdown";
import {
  DanswerDocument,
  FilteredDanswerDocument,
} from "@/lib/search/interfaces";
import { SearchSummary } from "./SearchSummary";
import { SkippedSearch } from "./SkippedSearch";
import remarkGfm from "remark-gfm";
import { CopyButton } from "@/components/CopyButton";
import {
  ChatFileType,
  FileDescriptor,
  ImageGenerationResults,
  ToolCallMetadata,
} from "../interfaces";
import {
  IMAGE_GENERATION_TOOL_NAME,
  SEARCH_TOOL_NAME,
  INTERNET_SEARCH_TOOL_NAME,
} from "../tools/constants";
import { ToolRunDisplay } from "../tools/ToolRunningAnimation";
import { Hoverable, HoverableIcon } from "@/components/Hoverable";
import { DocumentPreview } from "../files/documents/DocumentPreview";
import { InMessageImage } from "../files/images/InMessageImage";
import { CodeBlock } from "./CodeBlock";
import rehypePrism from "rehype-prism-plus";

import "prismjs/themes/prism-tomorrow.css";
import "./custom-code-styles.css";
import { Persona } from "@/app/admin/assistants/interfaces";
import { AssistantIcon } from "@/components/assistants/AssistantIcon";
import { Citation } from "@/components/search/results/Citation";

import {
  LikeFeedback,
  DislikeFeedback,
  ToolCallIcon,
} from "@/components/icons/icons";
import {
  CustomTooltip,
  TooltipGroup,
} from "@/components/tooltip/CustomTooltip";
import { ValidSources } from "@/lib/types";
import { Tooltip } from "@/components/tooltip/Tooltip";
import { useMouseTracking } from "./hooks";
import { SettingsContext } from "@/components/settings/SettingsProvider";
import GeneratingImageDisplay from "../tools/GeneratingImageDisplay";
import RegenerateOption from "../RegenerateOption";
import { LlmOverride } from "@/lib/hooks";
import { ContinueGenerating } from "./ContinueMessage";
import DualPromptDisplay from "../tools/ImagePromptCitaiton";
import { PopupSpec } from "@/components/admin/connectors/Popup";
import { Popover } from "@/components/popover/Popover";
import CsvPage from "@/components/chat_display/CsvDisplay";
import { DISABLED_CSV_DISPLAY } from "@/lib/constants";
import {
  LineChartDisplay,
  ModalChartWrapper,
} from "../../../components/chat_display/graphs/LineChartDisplay";
import PolarChartDisplay from "@/components/chat_display/graphs/PortalChart";
import BarChartDisplay from "@/components/chat_display/graphs/BarChart";

const TOOLS_WITH_CUSTOM_HANDLING = [
  SEARCH_TOOL_NAME,
  INTERNET_SEARCH_TOOL_NAME,
  IMAGE_GENERATION_TOOL_NAME,
];
import plotDataJson from "./linechart.json";
import barChartDataJson from "./barchart_data.json";
import polarChartDataJson from "./polar_plot_data.json";
import { JSONUpload } from "./JSONUpload";
import { ImageDisplay } from "@/components/chat_display/graphs/ImageDisplay";
import { InternetSearchIcon } from "@/components/InternetSearchIcon";
import { DocumentMetadataBlock } from "@/components/search/DocumentDisplay";

function FileDisplay({
  files,
  alignBubble,
}: {
  files: FileDescriptor[];
  alignBubble?: boolean;
}) {
  const imageFiles = files.filter((file) => file.type === ChatFileType.IMAGE);
  const nonImgFiles = files.filter(
    (file) => file.type !== ChatFileType.IMAGE && file.type !== ChatFileType.CSV
  );
  const csvImgFiles = files.filter((file) => file.type == ChatFileType.CSV);

  const [close, setClose] = useState(true);
  return (
    <>
      {nonImgFiles && nonImgFiles.length > 0 && (
        <div
          id="danswer-file"
          className={` ${alignBubble && "ml-auto"} mt-2 auto mb-4`}
        >
          <div className="flex flex-col gap-2">
            {nonImgFiles.map((file) => {
              return (
                <div key={file.id} className="w-fit">
                  <DocumentPreview
                    fileName={file.name || file.id}
                    maxWidth="max-w-64"
                    alignBubble={alignBubble}
                  />
                </div>
              );
            })}
          </div>
        </div>
      )}
      {csvImgFiles && csvImgFiles.length > 0 && (
        <div className={` ${alignBubble && "ml-auto"} mt-2 auto mb-4`}>
          <div className="flex flex-col gap-2">
            {csvImgFiles.map((file) => {
              return (
                <div key={file.id} className="w-fit">
                  {close && !DISABLED_CSV_DISPLAY ? (
                    <CsvPage
                      close={() => setClose(false)}
                      csvFileDescriptor={file}
                    />
                  ) : (
                    <DocumentPreview
                      open={
                        DISABLED_CSV_DISPLAY ? undefined : () => setClose(true)
                      }
                      fileName={file.name || file.id}
                      maxWidth="max-w-64"
                      alignBubble={alignBubble}
                    />
                  )}
                </div>
              );
            })}
          </div>
        </div>
      )}
      {/* <LineChartDisplay /> */}
      {imageFiles && imageFiles.length > 0 && (
        <div
          id="danswer-image"
          className={` ${alignBubble && "ml-auto"} mt-2 auto mb-4`}
        >
          <div className="flex flex-col gap-2">
            {imageFiles.map((file) => {
              return <InMessageImage key={file.id} fileId={file.id} />;
            })}
          </div>
        </div>
      )}
    </>
  );
}
export interface graph {
  file_id: string;
  line: boolean;
}

export interface GraphChunk {
  file_id: string;
  line_graph: boolean;
}
export const AIMessage = ({
  hasChildAI,
  hasParentAI,
  regenerate,
  overriddenModel,
  continueGenerating,
  shared,
  isActive,
  toggleDocumentSelection,
  alternativeAssistant,
  docs,
  graphs = [],
  messageId,
  content,
  files,
  selectedDocuments,
  query,
  citedDocuments,
  toolCall,
  isComplete,
  hasDocs,
  handleFeedback,
  isCurrentlyShowingRetrieved,
  handleShowRetrieved,
  handleSearchQueryEdit,
  handleForceSearch,
  retrievalDisabled,
  currentPersona,
  otherMessagesCanSwitchTo,
  onMessageSelection,
  setPopup,
}: {
  shared?: boolean;
  hasChildAI?: boolean;
  hasParentAI?: boolean;
  graphs?: graph[];
  isActive?: boolean;
  continueGenerating?: () => void;
  otherMessagesCanSwitchTo?: number[];
  onMessageSelection?: (messageId: number) => void;
  selectedDocuments?: DanswerDocument[] | null;
  toggleDocumentSelection?: () => void;
  docs?: DanswerDocument[] | null;
  alternativeAssistant?: Persona | null;
  currentPersona: Persona;
  messageId: number | null;
  content: string | JSX.Element;
  files?: FileDescriptor[];
  query?: string;
  citedDocuments?: [string, DanswerDocument][] | null;
  toolCall?: ToolCallMetadata | null;
  isComplete?: boolean;
  hasDocs?: boolean;
  handleFeedback?: (feedbackType: FeedbackType) => void;
  isCurrentlyShowingRetrieved?: boolean;
  handleShowRetrieved?: (messageNumber: number | null) => void;
  handleSearchQueryEdit?: (query: string) => void;
  handleForceSearch?: () => void;
  retrievalDisabled?: boolean;
  overriddenModel?: string;
  regenerate?: (modelOverRide: LlmOverride) => Promise<void>;
  setPopup?: (popupSpec: PopupSpec | null) => void;
}) => {
  const [isPopoverOpen, setIsPopoverOpen] = useState(false);

  const toolCallGenerating = toolCall && !toolCall.tool_result;
  const processContent = (content: string | JSX.Element) => {
    if (typeof content !== "string") {
      return content;
    }

    const codeBlockRegex = /```(\w*)\n[\s\S]*?```|```[\s\S]*?$/g;
    const matches = content.match(codeBlockRegex);

    if (matches) {
      content = matches.reduce((acc, match) => {
        if (!match.match(/```\w+/)) {
          return acc.replace(match, match.replace("```", "```plaintext"));
        }
        return acc;
      }, content);

      const lastMatch = matches[matches.length - 1];
      if (!lastMatch.endsWith("```")) {
        return content;
      }
    }
    if (
      isComplete &&
      toolCall?.tool_result &&
      toolCall.tool_name == IMAGE_GENERATION_TOOL_NAME
    ) {
      return content + ` [${toolCall.tool_name}]()`;
    }

    return content + (!isComplete && !toolCallGenerating ? " [*]() " : "");
  };
  const finalContent = processContent(content as string);

  const [isRegenerateHovered, setIsRegenerateHovered] = useState(false);

  const { isHovering, trackedElementRef, hoverElementRef } = useMouseTracking();

  const settings = useContext(SettingsContext);
  // this is needed to give Prism a chance to load

  const selectedDocumentIds =
    selectedDocuments?.map((document) => document.document_id) || [];
  let citedDocumentIds: string[] = [];

  citedDocuments?.forEach((doc) => {
    citedDocumentIds.push(doc[1].document_id);
  });

  if (!isComplete) {
    const trimIncompleteCodeSection = (
      content: string | JSX.Element
    ): string | JSX.Element => {
      if (typeof content === "string") {
        const pattern = /```[a-zA-Z]+[^\s]*$/;
        const match = content.match(pattern);
        if (match && match.index && match.index > 3) {
          const newContent = content.slice(0, match.index - 3);
          return newContent;
        }
        return content;
      }
      return content;
    };
    content = trimIncompleteCodeSection(content);
  }

  let filteredDocs: FilteredDanswerDocument[] = [];

  if (docs) {
    filteredDocs = docs
      .filter(
        (doc, index, self) =>
          doc.document_id &&
          doc.document_id !== "" &&
          index === self.findIndex((d) => d.document_id === doc.document_id)
      )
      .filter((doc) => {
        return citedDocumentIds.includes(doc.document_id);
      })
      .map((doc: DanswerDocument, ind: number) => {
        return {
          ...doc,
          included: selectedDocumentIds.includes(doc.document_id),
        };
      });
  }

  const currentMessageInd = messageId
    ? otherMessagesCanSwitchTo?.indexOf(messageId)
    : undefined;
  const uniqueSources: ValidSources[] = Array.from(
    new Set((docs || []).map((doc) => doc.source_type))
  ).slice(0, 3);

  const includeMessageSwitcher =
    currentMessageInd !== undefined &&
    onMessageSelection &&
    otherMessagesCanSwitchTo &&
    otherMessagesCanSwitchTo.length > 1;

  return (
    <div
      id="danswer-ai-message"
      ref={trackedElementRef}
      className={`${hasParentAI ? "pb-5" : "py-5"} px-2 lg:px-5 relative flex `}
    >
      <div
        className={`mx-auto ${shared ? "w-full" : "w-[90%]"}  max-w-message-max`}
      >
        <div className={`desktop:mr-12 ${!shared && "mobile:ml-0 md:ml-8"}`}>
          <div className="flex">
            {!hasParentAI ? (
              <AssistantIcon
                size="small"
                assistant={alternativeAssistant || currentPersona}
              />
            ) : (
              <div className="w-6" />
            )}

            <div className="w-full">
              <div className="max-w-message-max break-words">
                <div className="w-full ml-4">
                  <div className="max-w-message-max break-words">
                    {(!toolCall || toolCall.tool_name === SEARCH_TOOL_NAME) && (
                      <>
                        {query !== undefined &&
                          handleShowRetrieved !== undefined &&
                          isCurrentlyShowingRetrieved !== undefined &&
                          !retrievalDisabled && (
                            <div className="mb-1">
                              <SearchSummary
                                docs={docs}
                                filteredDocs={filteredDocs}
                                query={query}
                                finished={
                                  toolCall?.tool_result != undefined ||
                                  isComplete!
                                }
                                toggleDocumentSelection={
                                  toggleDocumentSelection
                                }
                                handleSearchQueryEdit={handleSearchQueryEdit}
                              />
                            </div>
                          )}
                        {handleForceSearch &&
                          !hasChildAI &&
                          content &&
                          query === undefined &&
                          !hasDocs &&
                          !retrievalDisabled && (
                            <div className="mb-1">
                              <SkippedSearch
                                handleForceSearch={handleForceSearch}
                              />
                            </div>
                          )}
                      </>
                    )}

                    {toolCall &&
                      !TOOLS_WITH_CUSTOM_HANDLING.includes(
                        toolCall.tool_name
                      ) && (
                        <ToolRunDisplay
                          toolName={
                            toolCall.tool_result && content
                              ? `Used "${toolCall.tool_name}"`
                              : `Using "${toolCall.tool_name}"`
                          }
                          toolLogo={
                            <FiTool size={15} className="my-auto mr-1" />
                          }
                          isRunning={!toolCall.tool_result || !content}
                        />
                      )}

                    {toolCall &&
                      (!files || files.length == 0) &&
                      toolCall.tool_name === IMAGE_GENERATION_TOOL_NAME &&
                      !toolCall.tool_result && <GeneratingImageDisplay />}

                    {toolCall &&
                      toolCall.tool_name === INTERNET_SEARCH_TOOL_NAME && (
                        <div className="my-2">
                          <ToolRunDisplay
                            toolName={
                              toolCall.tool_result
                                ? `Searched the internet`
                                : `Searching the internet`
                            }
                            toolLogo={
                              <FiGlobe size={15} className="my-auto mr-1" />
                            }
                            isRunning={!toolCall.tool_result}
                          />
                        </div>
                      )}
                    {graphs.map((graph, ind) => {
                      return graph.line ? (
                        <ModalChartWrapper
                          key={ind}
                          chartType="line"
                          fileId={graph.file_id}
                        >
                          <LineChartDisplay fileId={graph.file_id} />
                        </ModalChartWrapper>
                      ) : (
                        <ModalChartWrapper
                          key={ind}
                          chartType="bar"
                          fileId={graph.file_id}
                        >
                          <BarChartDisplay fileId={graph.file_id} />
                        </ModalChartWrapper>
                      );
                    })}

                    {content || files ? (
                      <>
                        <FileDisplay files={files || []} />

                        {typeof content === "string" ? (
                          <div className="overflow-x-visible max-w-content-max">
                            <ReactMarkdown
                              key={messageId}
                              className="prose max-w-full text-base"
                              components={{
                                a: (props) => {
                                  const { node, ...rest } = props;
                                  const value = rest.children;

                                  if (
                                    value
                                      ?.toString()
                                      .startsWith(IMAGE_GENERATION_TOOL_NAME)
                                  ) {
                                    const imageGenerationResult =
                                      toolCall?.tool_result as ImageGenerationResults;

                                    return (
                                      <Popover
                                        open={isPopoverOpen}
                                        onOpenChange={
                                          () => null
                                          // setIsPopoverOpen(isPopoverOpen => !isPopoverOpen)
                                        } // only allow closing from the icon
                                        content={
                                          <button
                                            onMouseDown={() => {
                                              setIsPopoverOpen(!isPopoverOpen);
                                            }}
                                          >
                                            <ToolCallIcon className="cursor-pointer flex-none text-blue-500 hover:text-blue-700 !h-4 !w-4 inline-block" />
                                          </button>
                                        }
                                        popover={
                                          <DualPromptDisplay
                                            arg="Prompt"
                                            setPopup={setPopup!}
                                            prompt1={
                                              imageGenerationResult[0]
                                                .revised_prompt
                                            }
                                            prompt2={
                                              imageGenerationResult[1]
                                                .revised_prompt
                                            }
                                          />
                                        }
                                        side="top"
                                        align="center"
                                      />
                                    );
                                  }

                                  if (value?.toString().startsWith("*")) {
                                    return (
                                      <div className="flex-none bg-background-800 inline-block rounded-full h-3 w-3 ml-2" />
                                    );
                                  } else if (
                                    value?.toString().startsWith("[")
                                  ) {
                                    return (
                                      <Citation link={rest?.href}>
                                        {rest.children}
                                      </Citation>
                                    );
                                  } else {
                                    return (
                                      <a
                                        onMouseDown={() =>
                                          rest.href
                                            ? window.open(rest.href, "_blank")
                                            : undefined
                                        }
                                        className="cursor-pointer text-link hover:text-link-hover"
                                      >
                                        {rest.children}
                                      </a>
                                    );
                                  }
                                },
                                code: (props) => (
                                  <CodeBlock
                                    className="w-full"
                                    {...props}
                                    content={content as string}
                                  />
                                ),
                                p: ({ node, ...props }) => (
                                  <p {...props} className="text-default" />
                                ),
                              }}
                              remarkPlugins={[remarkGfm]}
                              rehypePlugins={[
                                [rehypePrism, { ignoreMissing: true }],
                              ]}
                            >
                              {finalContent as string}
                            </ReactMarkdown>
                          </div>
                        ) : (
                          content
                        )}
                      </>
                    ) : isComplete ? null : (
                      <></>
                    )}
                  </div>
                  {/* <ModalChartWrapper chartType="line" fileId="fee2ff90-4ebe-43fc-858f-a95c73385da4" >
                        <LineChartDisplay fileId="fee2ff90-4ebe-43fc-858f-a95c73385da4" />
                      </ModalChartWrapper> */}
                  {/* 
                      <ModalChartWrapper chartType="bar" fileId={"0ad36971-9353-42de-b89d-9c3361d3c3eb"}>
                        <BarChartDisplay fileId={"0ad36971-9353-42de-b89d-9c3361d3c3eb"} />
                      </ModalChartWrapper>

                      <ModalChartWrapper chartType="other" fileId={"066fc31f-56f0-48fb-98d3-ffd46f1ac0f5"}>
                        <ImageDisplay fileId={"066fc31f-56f0-48fb-98d3-ffd46f1ac0f5"} />
                      </ModalChartWrapper>
                  */}

                  {!hasChildAI &&
                    handleFeedback &&
                    (isActive ? (
                      <div
                        className={`
                        flex md:flex-row gap-x-0.5 mt-1
                        transition-transform duration-300 ease-in-out
                        transform opacity-100 translate-y-0"
                  `}
                      >
                        <TooltipGroup>
                          <div className="flex justify-start w-full gap-x-0.5">
                            {includeMessageSwitcher && (
                              <div className="-mx-1 mr-auto">
                                <MessageSwitcher
                                  currentPage={currentMessageInd + 1}
                                  totalPages={otherMessagesCanSwitchTo.length}
                                  handlePrevious={() => {
                                    onMessageSelection(
                                      otherMessagesCanSwitchTo[
                                        currentMessageInd - 1
                                      ]
                                    );
                                  }}
                                  handleNext={() => {
                                    onMessageSelection(
                                      otherMessagesCanSwitchTo[
                                        currentMessageInd + 1
                                      ]
                                    );
                                  }}
                                />
                              </div>
                            )}
                          </div>
                          <CustomTooltip showTick line content="Copy!">
                            <CopyButton content={content.toString()} />
                          </CustomTooltip>
                          <CustomTooltip showTick line content="Good response!">
                            <HoverableIcon
                              icon={<LikeFeedback />}
                              onClick={() => handleFeedback("like")}
                            />
                          </CustomTooltip>
                          <CustomTooltip showTick line content="Bad response!">
                            <HoverableIcon
                              icon={<DislikeFeedback size={16} />}
                              onClick={() => handleFeedback("dislike")}
                            />
                          </CustomTooltip>
                          {regenerate && (
                            <RegenerateOption
                              onHoverChange={setIsRegenerateHovered}
                              selectedAssistant={currentPersona!}
                              regenerate={regenerate}
                              overriddenModel={overriddenModel}
                            />
                          )}
                        </TooltipGroup>
                      </div>
                    ) : (
                      <div
                        ref={hoverElementRef}
                        className={`
                        absolute -bottom-5
                        z-20
                        invisible ${(isHovering || isRegenerateHovered || settings?.isMobile) && "!visible"}
                        opacity-0 ${(isHovering || isRegenerateHovered || settings?.isMobile) && "!opacity-100"}
                        translate-y-2 ${(isHovering || settings?.isMobile) && "!translate-y-0"}
                        transition-transform duration-300 ease-in-out 
                        flex md:flex-row gap-x-0.5 bg-background-125/40 -mx-1.5 p-1.5 rounded-lg
                        `}
                      >
                        <TooltipGroup>
                          <div className="flex justify-start w-full gap-x-0.5">
                            {includeMessageSwitcher && (
                              <div className="-mx-1 mr-auto">
                                <MessageSwitcher
                                  currentPage={currentMessageInd + 1}
                                  totalPages={otherMessagesCanSwitchTo.length}
                                  handlePrevious={() => {
                                    onMessageSelection(
                                      otherMessagesCanSwitchTo[
                                        currentMessageInd - 1
                                      ]
                                    );
                                  }}
                                  handleNext={() => {
                                    onMessageSelection(
                                      otherMessagesCanSwitchTo[
                                        currentMessageInd + 1
                                      ]
                                    );
                                  }}
                                />
                              </div>
                            )}
                          </div>
                          <CustomTooltip showTick line content="Copy!">
                            <CopyButton content={content.toString()} />
                          </CustomTooltip>

                          <CustomTooltip showTick line content="Good response!">
                            <HoverableIcon
                              icon={<LikeFeedback />}
                              onClick={() => handleFeedback("like")}
                            />
                          </CustomTooltip>

                          <CustomTooltip showTick line content="Bad response!">
                            <HoverableIcon
                              icon={<DislikeFeedback size={16} />}
                              onClick={() => handleFeedback("dislike")}
                            />
                          </CustomTooltip>
                          {regenerate && (
                            <RegenerateOption
                              selectedAssistant={currentPersona!}
                              regenerate={regenerate}
                              overriddenModel={overriddenModel}
                              onHoverChange={setIsRegenerateHovered}
                            />
                          )}
                        </TooltipGroup>
                      </div>
                    ))}
                </div>
              </div>
            </div>
          </div>
        </div>
        {(!toolCall || toolCall.tool_name === SEARCH_TOOL_NAME) &&
          !query &&
          continueGenerating && (
            <ContinueGenerating handleContinueGenerating={continueGenerating} />
          )}
      </div>
    </div>
  );
};

function MessageSwitcher({
  currentPage,
  totalPages,
  handlePrevious,
  handleNext,
}: {
  currentPage: number;
  totalPages: number;
  handlePrevious: () => void;
  handleNext: () => void;
}) {
  return (
    <div className="flex items-center text-sm space-x-0.5">
      <Hoverable
        icon={FiChevronLeft}
        onClick={currentPage === 1 ? undefined : handlePrevious}
      />

      <span className="text-emphasis select-none">
        {currentPage} / {totalPages}
      </span>

      <Hoverable
        icon={FiChevronRight}
        onClick={currentPage === totalPages ? undefined : handleNext}
      />
    </div>
  );
}

export const HumanMessage = ({
  content,
  files,
  messageId,
  otherMessagesCanSwitchTo,
  onEdit,
  onMessageSelection,
  shared,
  stopGenerating = () => null,
}: {
  shared?: boolean;
  content: string;
  files?: FileDescriptor[];
  messageId?: number | null;
  otherMessagesCanSwitchTo?: number[];
  onEdit?: (editedContent: string) => void;
  onMessageSelection?: (messageId: number) => void;
  stopGenerating?: () => void;
}) => {
  const textareaRef = useRef<HTMLTextAreaElement>(null);

  const [isHovered, setIsHovered] = useState(false);
  const [isEditing, setIsEditing] = useState(false);
  const [editedContent, setEditedContent] = useState(content);

  useEffect(() => {
    if (!isEditing) {
      setEditedContent(content);
    }
  }, [content]);

  useEffect(() => {
    if (textareaRef.current) {
      // Focus the textarea
      textareaRef.current.focus();
      // Move the cursor to the end of the text
      textareaRef.current.selectionStart = textareaRef.current.value.length;
      textareaRef.current.selectionEnd = textareaRef.current.value.length;
      textareaRef.current.style.height = `${textareaRef.current.scrollHeight}px`;
    }
  }, [isEditing]);

  const handleEditSubmit = () => {
    if (editedContent.trim() !== content.trim()) {
      onEdit?.(editedContent);
    }
    setIsEditing(false);
  };

  const currentMessageInd = messageId
    ? otherMessagesCanSwitchTo?.indexOf(messageId)
    : undefined;

  return (
    <div
      id="danswer-human-message"
      className="pt-5 pb-1 px-2 lg:px-5 flex -mr-6 relative"
      onMouseEnter={() => setIsHovered(true)}
      onMouseLeave={() => setIsHovered(false)}
    >
      <div className={`mx-auto ${shared ? "w-full" : "w-[90%]"} max-w-[790px]`}>
        <div className="xl:ml-8">
          <div className="flex flex-col mr-4">
            <FileDisplay alignBubble files={files || []} />

            <div className="flex justify-end">
              <div className="w-full ml-8 flex w-full w-[800px] break-words">
                {isEditing ? (
                  <div className="w-full">
                    <div
                      className={`
                      opacity-100
                      w-full
                      flex
                      flex-col
                      border 
                      border-border 
                      rounded-lg 
                      bg-background-emphasis 
                      pb-2
                      [&:has(textarea:focus)]::ring-1
                      [&:has(textarea:focus)]::ring-black
                    `}
                    >
                      <textarea
                        ref={textareaRef}
                        className={`
                        m-0 
                        w-full 
                        h-auto
                        shrink
                        border-0
                        rounded-lg 
                        overflow-y-hidden
                        bg-background-emphasis 
                        whitespace-normal 
                        break-word
                        overscroll-contain
                        outline-none 
                        placeholder-gray-400 
                        resize-none
                        pl-4
                        overflow-y-auto
                        pr-12 
                        py-4`}
                        aria-multiline
                        role="textarea"
                        value={editedContent}
                        style={{ scrollbarWidth: "thin" }}
                        onChange={(e) => {
                          setEditedContent(e.target.value);
                          textareaRef.current!.style.height = "auto";
                          e.target.style.height = `${e.target.scrollHeight}px`;
                        }}
                        onKeyDown={(e) => {
                          if (e.key === "Escape") {
                            e.preventDefault();
                            setEditedContent(content);
                            setIsEditing(false);
                          }
                          // Submit edit if "Command Enter" is pressed, like in ChatGPT
                          if (e.key === "Enter" && e.metaKey) {
                            handleEditSubmit();
                          }
                        }}
                      />
                      <div className="flex justify-end mt-2 gap-2 pr-4">
                        <button
                          className={`
                          w-fit
                          bg-accent 
                          text-inverted 
                          text-sm
                          rounded-lg 
                          inline-flex 
                          items-center 
                          justify-center 
                          flex-shrink-0 
                          font-medium 
                          min-h-[38px]
                          py-2
                          px-3
                          hover:bg-accent-hover
                        `}
                          onClick={handleEditSubmit}
                        >
                          Submit
                        </button>
                        <button
                          className={`
                          inline-flex 
                          items-center 
                          justify-center 
                          flex-shrink-0 
                          font-medium 
                          min-h-[38px] 
                          py-2 
                          px-3 
                          w-fit 
                          bg-hover
                          bg-background-strong 
                          text-sm
                          rounded-lg
                          hover:bg-hover-emphasis
                        `}
                          onClick={() => {
                            setEditedContent(content);
                            setIsEditing(false);
                          }}
                        >
                          Cancel
                        </button>
                      </div>
                    </div>
                  </div>
                ) : typeof content === "string" ? (
                  <>
                    <div className="ml-auto mr-1 my-auto">
                      {onEdit &&
                      isHovered &&
                      !isEditing &&
                      (!files || files.length === 0) ? (
                        <Tooltip delayDuration={1000} content={"Edit message"}>
                          <button
                            className="hover:bg-hover p-1.5 rounded"
                            onClick={() => {
                              setIsEditing(true);
                              setIsHovered(false);
                            }}
                          >
                            <FiEdit2 className="!h-4 !w-4" />
                          </button>
                        </Tooltip>
                      ) : (
                        <div className="w-7" />
                      )}
                    </div>

                    <div
                      className={`${
                        !(
                          onEdit &&
                          isHovered &&
                          !isEditing &&
                          (!files || files.length === 0)
                        ) && "ml-auto"
                      } relative flex-none max-w-[70%] mb-auto whitespace-break-spaces rounded-3xl bg-user px-5 py-2.5`}
                    >
                      {content}
                    </div>
                  </>
                ) : (
                  <>
                    {onEdit &&
                    isHovered &&
                    !isEditing &&
                    (!files || files.length === 0) ? (
                      <div className="my-auto">
                        <Hoverable
                          icon={FiEdit2}
                          onClick={() => {
                            setIsEditing(true);
                            setIsHovered(false);
                          }}
                        />
                      </div>
                    ) : (
                      <div className="h-[27px]" />
                    )}
                    <div className="ml-auto rounded-lg p-1">{content}</div>
                  </>
                )}
              </div>
            </div>
          </div>

          {/* <CSVGraph /> */}
          {/* <CSVGraph fileId="./data.csv" /> */}

          <div className="flex flex-col md:flex-row gap-x-0.5 mt-1">
            {currentMessageInd !== undefined &&
              onMessageSelection &&
              otherMessagesCanSwitchTo &&
              otherMessagesCanSwitchTo.length > 1 && (
                <div className="ml-auto mr-3">
                  <MessageSwitcher
                    currentPage={currentMessageInd + 1}
                    totalPages={otherMessagesCanSwitchTo.length}
                    handlePrevious={() => {
                      stopGenerating();
                      onMessageSelection(
                        otherMessagesCanSwitchTo[currentMessageInd - 1]
                      );
                    }}
                    handleNext={() => {
                      stopGenerating();
                      onMessageSelection(
                        otherMessagesCanSwitchTo[currentMessageInd + 1]
                      );
                    }}
                  />
                </div>
              )}
          </div>
        </div>
      </div>
    </div>
  );
};
